/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef PECUNIA_SIZES_HPP
#define PECUNIA_SIZES_HPP

#include <cstdint>


namespace pecunia
{

/// @brief The type used when working with the currency minor unit.
using MinorUnit = std::uint16_t;
/// @brief The type used when working with the currency major unit. Also, the storage for the major,
/// minor, and sub-minor combined together.
using MajorUnit = std::int64_t;
/// @brief The base type used for all floating point operations.
using FloatingPointBase = double;
/// The number of digits to round at as the default.
inline constexpr std::uint8_t defaultRoundingDigits{3u};
/// The number of extra digits to have in the minor unit to use as guards when performing
/// mathematical operations.
inline constexpr std::uint8_t subMinorDigits{2u};
/// The default locale used within the library.
inline constexpr const char* defaultLocale{"C.UTF-8"};

}

#endif
