/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "Conversion.h"
using pecunia::MinorUnit;
using std::uint8_t;

#include <string>
using namespace std::string_literals;
using std::string;
using std::to_string;

#include "Codes.h"
using pecunia::currency::Currency;
using pecunia::currency::minorUnitDigits;
using pecunia::currency::toStdString;
#include "Errors.h"
using pecunia::OverflowError;
#include "FloatingPoint.h"
using namespace pecunia::literals;


MinorUnit pecunia::toMinorUnit(const MinorUnit& minor, const Currency& code)
{
	const auto numberMinorOnlyDigits{minorUnitDigits(code)};
	const auto conversionFactor{10.0_fp .power(numberMinorOnlyDigits)};

	if (minor > conversionFactor || conversionFactor.isEqual(minor))
		throw OverflowError{
			"The minor unit value "s + to_string(minor) + " is too large for the currency '"s
				+ toStdString(code) + "'."s
		};
	return static_cast<MinorUnit>((minor * conversionFactor).underlying());
}
