/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef PECUNIA_MANIP_H
#define PECUNIA_MANIP_H

#include <iomanip>

#include <pecunia/pecunia_export.h>


namespace pecunia::currency
{

/**
 * @brief A stream manipulator that reads and writes a currency indicator before the value.
 *
 * @param[in,out] stream The stream into which the manipulation is done.
 *
 * @return The manipulated stream.
 */
[[nodiscard]]
PECUNIA_EXPORT std::ios_base& leading(std::ios_base& stream);
/**
 * @brief A stream manipulator that reads and writes a currency indicator after the value.
 *
 * @param[in,out] stream The stream into which the manipulation is done.
 *
 * @return The manipulated stream.
 */
[[nodiscard]]
PECUNIA_EXPORT std::ios_base& trailing(std::ios_base& stream);
/**
 * @brief A stream manipulator that designates that the currency's symbol should be used when
 * writing.
 *
 * @param[in,out] stream The stream into which the manipulation is done.
 *
 * @return The manipulated stream.
 */
[[nodiscard]]
PECUNIA_EXPORT std::ios_base& symbol(std::ios_base& stream);
/**
 * @brief A stream manipulator that designates that the currency's ISO code should be used when
 * writing.
 *
 * @param[in,out] stream The stream into which the manipulation is done.
 *
 * @return The manipulated stream.
 */
[[nodiscard]]
PECUNIA_EXPORT std::ios_base& isocode(std::ios_base& stream);
/**
 * @brief A stream manipulator that places a space between the currency and its value when writing.
 *
 * @param[in,out] stream The stream into which the manipulation is done.
 *
 * @return The manipulated stream.
 */
[[nodiscard]]
PECUNIA_EXPORT std::ios_base& spaced(std::ios_base& stream);
/**
 * @brief A stream manipulator that places a no spaces between the currency and its value when
 * writing.
 *
 * @param[in,out] stream The stream into which the manipulation is done.
 *
 * @return The manipulated stream.
 */
[[nodiscard]]
PECUNIA_EXPORT std::ios_base& nonspaced(std::ios_base& stream);

}

#endif
