/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "Math.h"
using pecunia::currency::Currency;
using pecunia::currency::Money;
using std::function;
using std::map;

#include <cmath>
using std::abs;
#include <numeric>
using std::accumulate;


Money pecunia::math::absoluteValue(const Money& money)
{
	auto copy{money};
	copy.amount_ = abs(copy.amount_);
	return copy;
}

void pecunia::math::internal::addToRunningTotals(
	map<Currency, Money>& runningTotals,
	const Money& amount,
	const function<Money(const Money &)>& adjustor
)
{
	if (runningTotals.count(amount.code()) == 0)
		runningTotals[amount.code()].assign(adjustor == nullptr ? amount : adjustor(amount));
	else
		runningTotals[amount.code()] += adjustor == nullptr ? amount : adjustor(amount);
}

Money pecunia::math::internal::accumulateRunningTotals(
	const Currency& accumulateCurrency,
	const map<Currency, Money>& totals
)
{
	const auto total{
		accumulate(
			totals.cbegin(),
			totals.cend(),
			Money{accumulateCurrency},
			[] (const auto& accumulator, const auto& currencyRunningTotals)
			{
				return accumulator + currencyRunningTotals.second;
			}
		)
	};
	return total;
}
