/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "SetUp.h"
using pecunia::ConversionFunction;
using pecunia::converter;
using pecunia::currency::setUpCurrency;
using pecunia::defaultConverter;
using std::string;

#include <cassert>
// using assert;
#include <cfenv>
using std::feclearexcept;
#include <clocale>
using std::setlocale;

#include <pecunia/Sizes.hpp>
using pecunia::defaultLocale;

#include "FloatingPoint.h"
using pecunia::FloatingPoint;


void pecunia::currency::setUpCurrency(const ConversionFunction& converterFunctor)
{
	setUpCurrency(converterFunctor, defaultLocale);
}

void pecunia::currency::setUpCurrency(const string& locale)
{
	setUpCurrency(&defaultConverter, locale);
}

void pecunia::currency::setUpCurrency(
	const ConversionFunction& converterFunctor,
	const string& locale
)
{
	assert(converterFunctor != nullptr && "The conversion functor must be a function.");
	setlocale(LC_ALL, locale.c_str());
	feclearexcept(FE_ALL_EXCEPT);
	converter = converterFunctor;
}
