/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef PECUNIA_INTERNAL_ADJUSTMENTS_H
#define PECUNIA_INTERNAL_ADJUSTMENTS_H

#include <cstdint>

#include <pecunia/Types.h>


namespace pecunia
{

class FloatingPoint;

}
namespace pecunia::currency
{

enum class Currency : std::uint16_t;
class Money;

}
namespace pecunia::currency::internal
{

[[nodiscard]]
MinorUnit fromMinorMinorOnly(const MinorUnit minorAndSub, const std::uint8_t subMinorUnitDigits);

[[nodiscard]]
MinorUnit fromMinorSubMinor(const MinorUnit minorAndSub, const std::uint8_t subMinorUnitDigits);

[[nodiscard]]
MajorUnit fromFloatingPointMajor(
	const pecunia::FloatingPoint& value,
	const pecunia::currency::Currency& code,
	const std::uint8_t subMinorUnitDigits,
	const pecunia::FloatingPointRounderFunction& rounder
);

[[nodiscard]]
MinorUnit fromFloatingPointMinor(
	const pecunia::FloatingPoint& value,
	const pecunia::currency::Currency& code,
	const std::uint8_t subMinorUnitDigits,
	const pecunia::FloatingPointRounderFunction& rounder
);

[[nodiscard]]
MinorUnit fromFloatingPointSubMinor(
	const pecunia::FloatingPoint& value,
	const pecunia::currency::Currency& code,
	const std::uint8_t subMinorUnitDigits,
	const pecunia::FloatingPointRounderFunction& rounder
);

[[nodiscard]]
MajorUnit normaliseAmount(
	const pecunia::currency::Money& from,
	const pecunia::currency::Currency& to
);

[[nodiscard]]
MajorUnit fromAmountMajor(const MajorUnit& amount, const pecunia::currency::Currency& code);

[[nodiscard]]
MinorUnit fromAmountMinor(const MajorUnit& amount, const pecunia::currency::Currency& code);

[[nodiscard]]
MinorUnit fromAmountMinorOnly(const MajorUnit& amount, const pecunia::currency::Currency& code);

[[nodiscard]]
MinorUnit fromAmountSubMinor(const MajorUnit& amount, const pecunia::currency::Currency& code);

/**
 * @brief Calculates the currency factor between the major unit and the minor unit accounting for
 * the sub-minor monetary unit as well. The precision is in terms of how many sub-minor
 * digits to add to the minor unit along with the currency's minor digits, e.g. USD starts with 2
 * minor digits and with 2 sub-minor digits would provide for a total precision of 4 digits.
 *
 * @param[in] code The country's monetary unit to calculate the ratio for.
 *
 * @return The multiplicative factor between the major and minor unit.
 */
[[nodiscard]]
MinorUnit minorUnitSubMinorFactor(const Currency& code) noexcept;

}

#endif
