/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "TraceCapture.h"
using pecunia::internal::TraceCapture;
using std::array;

#include <algorithm>
using std::copy;
#include <utility>
using std::move;

#include <pecunia/Sizes.hpp>
using pecunia::internal::errorStackFrameSize;


TraceCapture::TraceCapture() noexcept :
	TraceCapture{{}, 0}
{}

TraceCapture::TraceCapture(array<void*, errorStackFrameSize> frames, int framesFilled) noexcept :
	frames_{move(frames)},
	framesFilled_{move(framesFilled)}
{}

TraceCapture::TraceCapture(const TraceCapture& other) noexcept :
	frames_{other.frames_},
	framesFilled_{other.framesFilled_}
{}

TraceCapture::TraceCapture(TraceCapture&& other) noexcept :
	frames_{move(other.frames_)},
	framesFilled_{move(other.framesFilled_)}
{}

TraceCapture::~TraceCapture() noexcept = default;

TraceCapture& TraceCapture::operator=(const TraceCapture& other) noexcept
{
	if (this != &other)
	{
		this->frames_ = other.frames_;
		this->framesFilled_ = other.framesFilled_;
	}
	return *this;
}

TraceCapture& TraceCapture::operator=(TraceCapture&& other) noexcept
{
	if (this != &other)
	{
		this->frames_ = move(other.frames_);
		this->framesFilled_ = move(other.framesFilled_);
	}
	return *this;
}

const array<void*, errorStackFrameSize>& TraceCapture::frames() const noexcept
{
	return this->frames_;
}

int TraceCapture::framesFilled() const noexcept
{
	return this->framesFilled_;
}
