/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef PECUNIA_INTERNAL_TRACECAPTURE_H
#define PECUNIA_INTERNAL_TRACECAPTURE_H

#include <array>

#include <internal/Sizes.hpp>


namespace pecunia::internal
{

class [[nodiscard]] TraceCapture
{
	std::array<void*, errorStackFrameSize> frames_;
	int framesFilled_;

public:
	TraceCapture() noexcept;
	TraceCapture(std::array<void*, errorStackFrameSize> frames, int framesFilled) noexcept;
	TraceCapture(const TraceCapture& other) noexcept;
	TraceCapture(TraceCapture&& other) noexcept;
	~TraceCapture() noexcept;
	TraceCapture& operator=(const TraceCapture& other) noexcept;
	TraceCapture& operator=(TraceCapture&& other) noexcept;

	[[nodiscard]]
	const std::array<void*, errorStackFrameSize>& frames() const noexcept;

	[[nodiscard]]
	int framesFilled() const noexcept;
};

}

#endif
