/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef PECUNIA_INTERNAL_VERIFICATION_H
#define PECUNIA_INTERNAL_VERIFICATION_H

#include <cstdint>

#include <pecunia/Types.h>


namespace pecunia
{

class FloatingPoint;

}
namespace pecunia::internal
{

enum class [[nodiscard]] VerifiedValue
{
	Fits,
	Overflows,
	Underflows,
	Undefined
};

VerifiedValue verifyAdditionFits(
	const MajorUnit& lhs,
	const MajorUnit& rhs,
	const MajorUnit& maximum,
	const MajorUnit& minimum
);
VerifiedValue verifySubtractionFits(
	const MajorUnit& lhs,
	const MajorUnit& rhs,
	const MajorUnit& maximum,
	const MajorUnit& minimum
);
VerifiedValue verifyMultiplicationFits(
	const MajorUnit& lhs,
	const FloatingPoint& rhs,
	const MajorUnit& maximum,
	const MajorUnit& minimum,
	const MinorUnit& minorMaximum,
	const std::uint8_t minorPrecision
);
VerifiedValue verifyDivisionFits(
	const MajorUnit& dividend,
	const FloatingPoint& divisor,
	const MajorUnit& maximum,
	const MajorUnit& minimum,
	const MinorUnit& minorMaximum,
	const std::uint8_t minorPrecision
);
VerifiedValue verifyMultiplicationFits(
	const MajorUnit& lhs,
	const MajorUnit& rhs,
	const MajorUnit& maximum,
	const MajorUnit& minimum
);
VerifiedValue verifyDivisionFits(
	const MajorUnit& dividend,
	const MajorUnit& divisor,
	const MajorUnit& minimum
);

}

#endif
