/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include <internal/BackTrace.h>
using namespace std::string_literals;
using std::string;
using std::vector;

#include <execinfo.h>
using ::backtrace;
using ::backtrace_symbols;

#include <array>
using std::array;
#include <cstddef>
using std::size_t;
#include <cstdlib>
using std::free;
#include <iostream>
using std::cerr;
#include <utility>
using std::move;

#include <internal/Sizes.hpp>
using pecunia::internal::errorStackFrameSize;
#include <internal/TraceCapture.h>
using pecunia::internal::TraceCapture;


TraceCapture pecunia::internal::captureBackTrace() noexcept
{
	array<void*, errorStackFrameSize> frames;
	int framesFilled{backtrace(frames.data(), errorStackFrameSize)};

	if (framesFilled >= errorStackFrameSize)
		cerr << "Captured back-trace may have been truncated.";
	TraceCapture trace{move(frames), framesFilled};
	return trace;
}

vector<string> pecunia::internal::generateBackTrace(const TraceCapture& trace)
{
	const auto backTraceLines{backtrace_symbols(trace.frames().data(), trace.framesFilled())};

	if (backTraceLines == nullptr)
		return {};
	constexpr size_t captureOffset{2}; // N.B. Skipping the capture functions.
	vector<string> lines{static_cast<size_t>(trace.framesFilled()) - captureOffset};

	for (size_t index{captureOffset} ; index < static_cast<size_t>(trace.framesFilled()) ; ++index)
	{
		string line{backTraceLines[index]};
		lines[index - captureOffset] = move(line);
	}
	free(backTraceLines);
	return lines;
}
